@@ -12,7 +12,7 @@ import numpy as np
 # https://github.com/EttusResearch/uhd/blob/master/host/lib/usrp/b200/b200_impl.hpp
 # Should result in no overflows:
 # UHD_IMAGES_DIR=/usr/share/uhd/images ./examples/rx_samples_to_file --args num_recv_frames=128,recv_frame_size=16360 --file test.gz --nsamps 200000000 --rate 20000000 --freq 101e6 --spb 20000000
-ETTUS_ARGS = 'num_recv_frames=128,recv_frame_size=16360'
+ETTUS_ARGS = 'num_recv_frames=768,recv_frame_size=16360'
 ETTUS_ANT = 'TX/RX'
 SAMPLE_FILENAME_RE = re.compile(r'^.+_([0-9]+)Hz_([0-9]+)sps\.(s\d+|raw).*$')
 SAMPLE_DTYPES = {
@@ -36,6 +36,10 @@ def replace_ext(filename, ext, all_ext=False):
     return filename.replace(basename, new_basename)
 
 
+def is_fft(filename):
+    return os.path.basename(filename).startswith('fft_')
+
+
 def parse_filename(filename):
     # TODO: parse from sigmf.
     match = SAMPLE_FILENAME_RE.match(filename)
@@ -47,6 +51,9 @@ def parse_filename(filename):
         freq_center = None
         sample_rate = None
         sample_type = None
+    # FFT is always float not matter the original sample type.
+    if is_fft(filename):
+        sample_type = 'raw'
     sample_dtype, sample_type = SAMPLE_DTYPES.get(sample_type, (None, None))
     sample_bits = None
     sample_len = None
