@@ -17,6 +17,10 @@ class UtilsTestCase(unittest.TestCase):
         self.assertEqual(
             (140000000, 20000000, np.dtype([('i', '<i2'), ('q', '<i2')]), 4, 'signed-integer', 16),
             parse_filename('gamutrf_recording1645540092_140000000Hz_20000000sps.s16.gz'))
+        self.assertEqual(
+            (140000000, 20000000, np.dtype([('i', '<f4'), ('q', '<f4')]), 8, 'float', 32),
+            parse_filename('fft_gamutrf_recording1645540092_140000000Hz_20000000sps.s16.gz'))
+
 
 
 if __name__ == '__main__':
